Scheduler = LCS.class(
	{
		storage = {}
	}
)

function Scheduler:init()
	
end

function Scheduler:update(dt)
	-- Add time to all animations and then execute their lambdas
	-- Their lambdas will return true if they should go on.
	-- Otherwise they return false and need to be deleted.
	-- dt is delta time.
	local cleanup = {}
	for animation_id, anim in ipairs(self.storage) do
		if not anim.paused then
			-- Update the current time counter for the animation
			-- We divide dt by anim.duration, since anim.current_time is always in [0, 1]
			anim.current_time = anim.current_time + dt / anim.duration

			-- Apply easing to the current time
			local current_eased_time = anim.easing(anim.current_time)

			-- Run the function
			continue = anim.fn(current_eased_time, current_eased_time - anim.easing(anim.last_time))

			-- Remember this as the previous time counter
			anim.last_time = anim.current_time

			if not continue then
				if anim.looping then
					anim.current_time = 0
				else
					table.insert(cleanup, animation_id)
					if anim.callback ~= nil then
						anim.callback()
					end
				end
			end
		end
	end

	-- Clean up if needed
	for i, a in ipairs(cleanup) do
		self:remove_by_id(a)
	end
end

function Scheduler:remove_by_id(id)
	-- Removes one animation from storage based on ID.
	-- You really shouldn't be doing this unless you know the ID.
	table.remove(self.storage, id)
end

function Scheduler:remove( animation )
	table.remove(self.storage, self:search(animation))
end

function Scheduler:add(target_lam, target_duration, target_easing, callback, looping)
	-- Where target_lam is the function of the animation and
	-- target_duration is the animation's duration in seconds
	-- last_time contains the last timestamp sent to the function.
	-- target_easing is the name of the easing function
	-- looping is a boolean. If true the event will not be cleaned up after it finishes.
	
	easing_lambda = animations.easings[target_easing] or animations.easings["linear"]
	
	local new_anim = {
		duration = target_duration,
		easing = easing_lambda,
		current_time = 0.0,
		last_time = 0.0,
		callback = callback,
		looping = looping
	}

	new_anim.fn = target_lam
	
	table.insert(self.storage, new_anim)
	
	return new_anim
end

function Scheduler:add_interval(interval, callback)
	-- TODO: implement this.
end

function Scheduler:pause(animation)
	animation.paused = true
end

function Scheduler:search(obj)
	-- Returns id of obj in the storage. Otherwise returns false.
	-- for i, a in ipairs(self.storage) do
	-- 	if a == obj then
	-- 		return i
	-- 	end
	-- end
	-- return false
	return table.search(self.storage, obj)
end

animations = {}

function animations.fadeSound(sound, volume_delta)
	-- Fade sounds 'n' scheiss
	local f = function(timestamp, delta)
		if timestamp >= 1 then
			sound:setVolume(sound:getVolume() + volume_delta * (1 - timestamp + delta))
			return false
		end
		sound:setVolume(sound:getVolume() + volume_delta * delta)

		return true
	end

	return f
end

function animations.moveLinear(obj, x_offset, y_offset)
	-- Move items linearly
	local f = function(timestamp, delta)
		if timestamp >= 1 then
			obj.x = obj.x + x_offset * (1 - timestamp + delta)
			obj.y = obj.y + y_offset * (1 - timestamp + delta)
			return false -- don't continue
		end
		obj.x = obj.x + x_offset * delta
		obj.y = obj.y + y_offset * delta

		return true
	end
	return f
end

function animations.rotate(obj, r)
	-- Rotate obj by r radians

	local f = function(timestamp, delta)
		if timestamp >= 1 then
			obj.facing = obj.facing + r * (1 - timestamp + delta)
			return false -- don't continue
		end
		obj.facing = obj.facing + r * delta
		return true
	end
	return f
end

function animations.delay(callback)
	-- Simple delay "animation" with a callback
	local f = function(timestamp, delta)
		if timestamp >= 1 then
			callback()
			return false
		end
		return true
	end
	return f
end

animations.easings = {
	linear = function(progression)
		return progression
	end,
	sin = function(progression)
		if progression >= 1 then
			return 1
		else
			return math.sin(progression * (math.pi / 2))
		end
	end
}